// ignore_for_file: avoid_print
import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:recase/recase.dart';
import 'package:path/path.dart' as path;
import 'package:yaml/yaml.dart';

void main(List<String> args) async {
  var location = Platform.script.toString();
  var isNewFlutter = location.contains(".snapshot");
  if (isNewFlutter) {
    var sp = Platform.script.toFilePath();
    var sd = sp.split(Platform.pathSeparator);
    sd.removeLast();
    var scriptDir = sd.join(Platform.pathSeparator);
    var packageConfigPath = [scriptDir, '..', '..', '..', 'package_config.json']
        .join(Platform.pathSeparator);
    // print(packageConfigPath);
    var jsonString = File(packageConfigPath).readAsStringSync();
    // print(jsonString);
    Map<String, dynamic> packages = jsonDecode(jsonString);
    var packageList = packages["packages"];
    String? ametoryUri;
    for (var package in packageList) {
      if (package["name"] == "ametory_framework") {
        ametoryUri = package["rootUri"];
        break;
      }
    }
    if (ametoryUri == null) {
      print("zoom package not found!");
      return;
    }
    if (ametoryUri.contains("../../")) {
      ametoryUri = ametoryUri.replaceFirst("../", "");
      ametoryUri = path.absolute(ametoryUri, "");
    }
    if (ametoryUri.contains("file:///")) {
      ametoryUri = ametoryUri.replaceFirst("file://", "");
      ametoryUri = path.absolute(ametoryUri, "");
    }
    location = ametoryUri;
  }

  if (args.isEmpty) {
    print("Please named new feature");
    return;
  }
  ReCase rc = ReCase(args.first);
  String appRootFolder = path.absolute("", "");

  // print(rc.camelCase);
  // print(rc.constantCase);
  // print(rc.snakeCase);
  // print(rc.pascalCase);
  // print(rc.paramCase);
  var pubFile = File("$appRootFolder/pubspec.yaml");
  var doc = loadYaml(pubFile.readAsStringSync());

  String appName = doc['name'];
  String libPath = "$location/lib";
  String templatePath = "$libPath/template/feature/example";
  String featureAppPath = "$appRootFolder/lib/feature/${rc.snakeCase}";
  String commonPath = "$libPath/template/common";
  String commonAppPath = "$appRootFolder/lib/common";
  String routerAppPath = "$commonAppPath/route";
  var list = await dirContents(Directory(templatePath));
  for (var element in list) {
    // CREATE DIRECTORY
    if (element.toString().contains("Directory")) {
      var newFolder =
          "$featureAppPath${element.path.replaceFirst(templatePath, "")}";
      if (!Directory(newFolder).existsSync()) {
        print("CREATE FOLDER => $newFolder");
        Directory(newFolder).createSync(recursive: true);
      }
    } else {
      String fileContent = await createFile(element.path, rc, appName);
      var newFile =
          "$featureAppPath${element.path.replaceFirst(templatePath, "").replaceAll("example", rc.snakeCase).replaceFirst(".stub", "")}";
      print("CREATE FILE => $newFile");
      File(newFile).writeAsStringSync(fileContent);
      // break;
    }
  }

  // APPEND ROUTER FILE

  String routerAppendContent =
      "static const String ${rc.camelCase} = '/${rc.snakeCase}';";
  // File("$commonPath/route/router.dart.stub").readAsStringSync();
  String routerFile = "$routerAppPath/router.dart";
  String routerFileContent = File(routerFile).readAsStringSync();
  String newFileRouterContent = routerFileContent.replaceAll(
      "// PLEASE DON'T REMOVE THIS LINE",
      "$routerAppendContent\n  // PLEASE DON'T REMOVE THIS LINE");
  String routerGeneratorAppendContent = """
case Routes.example:
        if (args is ExampleArguments) {
          return PageTransition(
              type: PageTransitionType.rightToLeft,
              child: ExamplePage(
                args: args,
              ));
        }
        return _errorRoute();
  """;
  routerGeneratorAppendContent = routerGeneratorAppendContent
      .replaceAll("example", rc.camelCase)
      .replaceAll("Example", rc.pascalCase);
  String routerGeneratorFile = "$routerAppPath/router_generator.dart";
  String routerGeneratorFileContent =
      File(routerGeneratorFile).readAsStringSync();

  String routeImportArgs =
      "import 'package:$appName/feature/${rc.snakeCase}/arguments/${rc.snakeCase}_arguments.dart';";
  String routeImportPage =
      "import 'package:$appName/feature/${rc.snakeCase}/ui/page/${rc.snakeCase}_page.dart';";
  String newFileRouterGeneratorContent = routerGeneratorFileContent.replaceAll(
      "// PLEASE DON'T REMOVE THIS LINE",
      "$routerGeneratorAppendContent\n      // PLEASE DON'T REMOVE THIS LINE");
  newFileRouterGeneratorContent = newFileRouterGeneratorContent.replaceFirst(
      "// PLEASE DON'T REMOVE THIS IMPORT LINE",
      "$routeImportArgs\n$routeImportPage\n// PLEASE DON'T REMOVE THIS IMPORT LINE");
  File(routerFile).writeAsStringSync(newFileRouterContent);
  File(routerGeneratorFile).writeAsStringSync(newFileRouterGeneratorContent);
}

Future<List<FileSystemEntity>> dirContents(Directory dir) {
  var files = <FileSystemEntity>[];
  var completer = Completer<List<FileSystemEntity>>();
  var lister = dir.list(recursive: true);
  lister.listen((file) => files.add(file),
      // should also register onError
      onDone: () => completer.complete(files));
  return completer.future;
}

Future<String> createFile(String path, ReCase rc, String appName) async {
  // snake_case
// dot.case
// path/case
// param-case
// PascalCase
// Header-Case
// Title Case
// camelCase
// Sentence case
// CONSTANT_CASE
  String result = File(path).readAsStringSync();
  result = result.replaceAll("appName", appName);
  result = result.replaceAll("Example", rc.pascalCase);
  result = result.replaceAll("example", rc.snakeCase);
  result = result.replaceAll("EXAMPLE", rc.constantCase);
  return result;
}
